# Version: @(#)dialog.tcl	1.19 2/7/94
#
#---------------------------------------------------------------------------
# Copyright (c) 1993 The Regents of the University of California.
# All rights reserved.
#
# Permission is hereby granted, without written agreement and without
# license or royalty fees, to use, copy, modify, and distribute this
# software and its documentation for any purpose, provided that the above
# copyright notice and the following two paragraphs appear in all copies
# of this software.
#
# IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
# FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
# ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
# THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
# PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
# CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
# ENHANCEMENTS, OR MODIFICATIONS.
#---------------------------------------------------------------------------
#
#  Programmers: Wei-Jen Huang <whuang@eecs.Berkeley.EDU>
#		Mario Jorge Silva <msilva@cs.Berkeley.EDU>
#
#  This file contains the file browser routines for opening facets.
#
#  The appearance of the browser is based largely on Mario Silva's
#	browser (fileselect.tcl) and Frame's "Open File" dialog.
#
#---------------------------------------------------------------------------
#
# Global variable dialogLastOpened is used to store the name of the
#  last facet opened
#
# Navigates through directories 

# This procedure is called to create the file browser.

proc dialog.openFacet {
    {w .fileBrowser}} {

    global env openDialogGlobal
    if {[winfo exists $w] && [winfo ismapped $w]} {
	wm deiconify $w
	raise $w
	return
    }
    catch {destroy $w}
    toplevel $w
    wm iconname $w "ptolemy"
    wm title $w "Ptolemy - Open Facet"

    if {! [info exists openDialogGlobal(dialogLastOpened)]} {
	# First call to this routine
	set openDialogGlobal(dialogLastOpened) init.pal
	set openDialogGlobal(CWD) $env(PTPWD)
	#set openDialogGlobal(CWD) [pwd]
    }

    pack append [frame $w.top -bd 5] \
	[label $w.top.label -text "Open Facet:"] {left top frame w} \
	[entry $w.top.entry -relief sunken -bd 3] \
		{top frame w fillx padx 3m pady 2m} \
	[frame $w.top.listscroll] {top expand fill} \
	[label $w.top.dir -textvar openDialogGlobal(CWD) \
		-width 35 -anchor e] {top fillx}

    listbox $w.top.listscroll.listbox -relief sunken \
		-yscroll "$w.top.listscroll.scroll set" -setgrid 1

    dialog.UpdateListbox $w openDialogGlobal CWD w

# In case dialog.Updateistbox inserts into the entry; to avoid repetition

    $w.top.entry delete 0 end
    $w.top.entry insert 0 $openDialogGlobal(dialogLastOpened)

# Update the listbox whenever the environment PWD variable is changed.
# [Don't refer to the environment PWD var, it does not exist on ultrix -cxh]
#    trace var openDialogGlobal w "dialog.UpdateListbox $w"

    pack append $w.top.listscroll \
	$w.top.listscroll.listbox {left expand fill} \
	[scrollbar $w.top.listscroll.scroll -command \
	"$w.top.listscroll.listbox yview" -relief sunken] {right filly}

    ptkOkCancelButtons [frame $w.bot -bd 1 -relief sunken] \
	"dialog.CheckFile $w \[$w.top.entry get\]" \
	"destroy $w"

    pack append $w $w.top {top expand fill} \
	$w.bot {bottom fill}

    tk_listboxSingleSelect $w.top.listscroll.listbox

    bind $w.top.listscroll.listbox <Double-1> \
	"$w.bot.f.ok config -relief sunken
	update idletasks
	dialog.CheckFile $w \[selection get\]
	if \[winfo exists $w.bot.f.ok\] {
		$w.bot.f.ok config -relief raised
	}"

# Update entry contents

    bind $w.top.listscroll.listbox <1> "
	%W select from \[%W nearest %y\]
	$w.top.entry delete 0 end
	$w.top.entry insert 0 \[selection get\]"

# Read entry contents

    ptkRecursiveBind $w <Return> "
	$w.bot.f.ok config -relief sunken
	update idletasks
	dialog.CheckFile $w \[$w.top.entry get\]
	if \[winfo exists $w.bot.f.ok\] {
		$w.bot.f.ok config -relief raised
	}"
# Extracted from tkerror.tcl Tk 3.3

    wm withdraw $w
    update idletasks
    set x [expr [winfo screenwidth $w]/2-[winfo reqwidth $w]/2-[
	    winfo vrootx [winfo parent $w]]]
    set y [expr [winfo screenheight $w]/2-[winfo reqheight $w]/2-[
	    winfo vrooty [winfo parent $w]]]
    wm geom $w +$x+$y
    wm deiconify $w

    focus $w.top.entry
}


# This procedure updates the listbox. It is triggered on writing to the
#  global 'openDialogGlobal' array.
# It classifies on basis of directories, 'facets' and files.
# Facets presently recognized by either the existence of a 
#  'schematic/contents;' file or the presence of an empty 'schematic'
#  directory.  Files are discarded before listing.
# Facets are inserted at the top and are arranged alphabetically.

proc dialog.UpdateListbox {w array idx op} {
    global openDialogGlobal
    if {$idx == "CWD"} {
	$w.top.listscroll.listbox delete 0 end
	# set files [glob -nocomplain "$cwd/{.,}*"]
	set files [glob -nocomplain $openDialogGlobal(CWD)/*] 
	set dirList ""
	set facetList ""
	foreach file $files {
	   set tail [file tail $file]
	   if [file isdir $file] {
                if [file exists "$file/schematic"] {
			lappend facetList $tail
		} else { lappend dirList $tail }
	   }
	}
	set dirList [lsort $dirList]
	set facetList [lsort $facetList]
	foreach facet $facetList {
		$w.top.listscroll.listbox insert end "$facet"}
	foreach dir $dirList {$w.top.listscroll.listbox insert end "$dir/"}
	$w.top.entry delete 0 end

	set dirLength [llength $dirList]
	set facetLength [llength $facetList]
	if {$openDialogGlobal(CWD) != "/"} {
		$w.top.listscroll.listbox insert 0 "../"
	}

	if {($dirLength == 1) && ($facetLength == 0)} {
		$w.top.entry insert 0 [lindex $dirList 0]
	} elseif {($dirLength == 0) && ($facetLength == 1)} {
		$w.top.entry insert 0 [lindex $facetList 0]
	} elseif {($dirLength == 0) && ($facetLength == 0)} {
		if {$openDialogGlobal(CWD) != "/"} {
			$w.top.entry insert 0 "../"
		}
	}
    }
}

# This procedure checks and processes the user selection.  Facets are
#  opened if they exist, and created if necessary when there is no
#  directory into which to change.

proc dialog.CheckFile {w args} {
    global openDialogGlobal
    set file [ptkExpandEnvVar [lindex $args 0]]
    if {$file == ""} {return}
    set file [string trimright [string trimleft $file]]

    # For use later _if_ necessary
    set relPath $file

    # Check for non-relative pathnames
    set char [string index $file 0]
    if {$char == "/" || $char == "~"} {
	set relative 0
    } else { set relative 1 }

    # Flag to indicate whether a new facet was created
    set newFacet 0
    if $relative {
	if {! [file exist $openDialogGlobal(CWD)/$file]} {
		pvOpenWindow [ptkOpenFacet $openDialogGlobal(CWD)/$file]
		set newFacet 1
	} else {
		set file $openDialogGlobal(CWD)/$file
	}
    } else {
	if {! [file exist $file]} {
		pvOpenWindow [ptkOpenFacet $file] 
		set newFacet 1
	}
    }
    if $newFacet {
	set openDialogGlobal(dialogLastOpened) $relPath
	destroy $w
#	trace vdel openDialogGlobal w "dialog.UpdateListbox $w"
	return
    } elseif {[file isdir $file]} {
	if {[file readable $file] && [file executable $file]} {
		set listing [glob -nocomplain "$file/schematic/*"]
		if {[file isdir $file/schematic] && \
		    ([file exists "$file/schematic/contents;"] || \
			$listing == "")}  {
		    if {[file readable $file/schematic] && \
			[file executable $file/schematic]} {
			pvOpenWindow [ptkOpenFacet $file]

			# Store last facet opened
			if $relative {
			   set openDialogGlobal(dialogLastOpened) \
				$relPath
			} else {
			   set openDialogGlobal(dialogLastOpened) $file
			}
			destroy $w
#			trace vdel openDialogGlobal w \
#				"dialog.UpdateListbox $w"
			return
		    } else {
			ptkImportantMessage .error \
				"\"$file/schematic\" is unreadable"
		    }
		} else {
		    # Non-relative path
		    if {$char == "~"} {
			set dir [glob -nocomplain $relPath]
			if {! [file exist $dir]} {
			  ptkImportantMessage .error \
				"\"$dir\" does not exist!"
			  return
			} else {
			  set openDialogGlobal(CWD) $dir
			  set openDialogGlobal(dialogLastOpened) ""
			  dialog.UpdateListbox $w openDialogGlobal CWD w
			  return
			}
		    }
		    if {$char == "/"} {
			set openDialogGlobal(CWD) $relPath
			set openDialogGlobal(dialogLastOpened) ""
			dialog.UpdateListbox $w openDialogGlobal CWD w
			return 
		    }
		    # Else relative path
		    set newPath $openDialogGlobal(CWD)
		    while {[regsub {^([^/]+)(/.*$|$)} $relPath {\1 \2} \
				temp]} {
			set partOfPath [lindex $temp 0]
			set relPath [string trimleft \
					[lindex $temp 1] "/"]
			if {$partOfPath == "."} {
				continue
			} elseif {$partOfPath == ".."} {
			   if [regsub {(.*)/[^/]+$} $newPath {\1} temp] {
			    set newPath $temp
			   } else {
			    ptkImportantMessage .error \
				"Invalid component: \"$partOfPath\""
			    return
			   }
			} else {
			    if {$newPath == "/"} {
				if [file exists /$partOfPath] {
					set newPath /$partOfPath
				} else {
				  ptkImportantMessage .error \
				   "Invalid component: \"$partOfPath\""
				  return
				}
			    } else {
			      if [file exists $newPath/$partOfPath] {
					set newPath $newPath/$partOfPath
				} else {
				  ptkImportantMessage .error \
				   "Invalid component: \"$partOfPath\""
				  return
				}
			    }
			}
		    }
		    if {$newPath == ""} {
			set openDialogGlobal(CWD) "/"
		    } else {set openDialogGlobal(CWD) $newPath}
		    set openDialogGlobal(dialogLastOpened) ""
		    dialog.UpdateListbox $w openDialogGlobal CWD w
		}
	} else {
		ptkImportantMessage .error \
		   "Directory \"[string trimright $file /]\" is unreadable"
	}
    }
}
